import {
  Singleton, Metadata, Router, Http, IEventParam, IDictionary, Config, dayjs as moment, Interceptor,
} from '@zeedhi/core';
import {
  Menu, DialogService, Select, AlertService,
} from '@zeedhi/common';
import { TekLibAuth } from '@zeedhi/tek-lib';
import { TekUserInfo } from '@zeedhi/teknisa-components-common';
import { IFavoritesItem, IFavoritesSaveEventParams } from '@zeedhi/zd-favorites-common';
import store from '../store';

@Singleton
export class AppController {
  public hasImage: boolean = false;

  get notHasImage(): boolean {
    return !this.hasImage;
  }
  
  get menuLeftSlot(): boolean {
    return store.state.appShowMenu;
  }

  public imagePath: string = '';
  
  public getPageName() {
    return document.title.split(' | ')[0];
  }

  private favorites: IFavoritesItem[] = [];

  public loadFavorites() {
    const storedFavorites = localStorage.getItem('favorites');
    if (storedFavorites) {
      this.favorites = JSON.parse(storedFavorites);
    }
    return this.favorites;
  }

  public saveFavorites({ favorites }: IFavoritesSaveEventParams) {
    if (favorites) {
      this.favorites = favorites;
      localStorage.setItem('favorites', JSON.stringify(favorites));
    } else {
      this.favorites = [];
      localStorage.setItem('favorites', JSON.stringify([]));
    }
  }

  public microservicePage: boolean = false;
  
  public headerElementsVisibility = true;

  public toggleMenu() {
    const menu = Metadata.getInstance<Menu>('main-menu');
    const isMobile = window.innerWidth <= menu.mobileBreakpoint;
  
    if (isMobile) {
      menu.drawer = !menu.drawer;
      menu.mini = false;
      menu.showSearch = true;
    } else {
      menu.drawer = true;
      menu.mini = !menu.mini;
      
      if (!this.isMini) {
        menu.showSearch = true;
        setTimeout(() => {
          const searchInput = document.querySelector('.zd-menu-search-input input');
          if (searchInput) (searchInput as HTMLElement).focus();
        }, 100);
      } else {
        menu.showSearch = false;
        const firstMenuItem = document.querySelector('.zd-menu-link, .zd-menu-group > div');
        if (firstMenuItem) (firstMenuItem as HTMLElement).focus();
      }
    }
  }

  // public confirmLogoutModal!: Modal;

  public metaLanguages: IDictionary = {
    pt_br: 'pt-BR',
    es_cl: 'es-CL',
    es_es: 'es-ES',
    es_co: 'es-CO',
    en_us: 'en-US',
  };

  public metaCurrency: IDictionary = {
    pt_br: 'BRL',
    es_cl: 'CLP',
    es_es: 'EUR',
    es_co: 'COP',
    en_us: 'USD',
  };

  public get nmCentral() {
    const central = Metadata.getInstance<Select>('CDFILCENTPME');
    return central.value ? (central.value as IDictionary).NMFILIAL : '';
  }
  
  public get cdCentral() {
    return store.getters.cdCentral;
  }

  get isMini() {
    try {
      const menu = Metadata.getInstance<Menu>('main-menu');
      if (menu.mini === true) menu.miniState = true;
      else menu.miniState = false;
      return menu.mini;
    } catch (error) {
      return true;
    }
  }
  
  get isNotMini() {
    return !this.isMini;
  }
  
  public routerAcessMini() {
    const menu = Metadata.getInstance<Menu>('main-menu');
    menu.showSearch = false;
    menu.mini = !menu.mini;
    menu.clearSearch();
  }
  
  public onInit() {
    Interceptor.addResponseSuccess((axiosConfig: any) => {
      if (axiosConfig.data.error) {
        return Promise.reject(axiosConfig.data.error);
      }
      
      return axiosConfig;
    });
  }
  
  /* private setLangDefault() {
    const language = localStorage.getItem('LOGIN_LANGUAGE');
    if (!language || language === 'undefined') {
      localStorage.setItem('LOGIN_LANGUAGE', 'pt_br');
    } else {
      I18n.changeLanguage(this.metaLanguages[language]);
    }
  } */
  
  public setUniDefault() {
    const fieldCentral = Metadata.getInstance<Select>('CDFILCENTPME');
    const currentRoute = Router.getCurrentRoute() as IDictionary;
    const loginRoute = Config.env.loginRoutePath;
    if (currentRoute.name !== loginRoute) {
      const actualCdCentral = localStorage.getItem('CDFILIAL');
      const msgParamCentral = () => {
        DialogService.show(
          {
            name: 'paramCentral',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::parametrizacaoCentral',
            buttons: [
              {
                name: 'dialogButton', label: 'pme::OK', component: 'ZdButton', events: { click: () => DialogService.hide() },
              },
            ],
          },
        );
      };
      
      if (actualCdCentral && actualCdCentral !== 'null') {
        Http.get('/central').then((response) => {
          Http.get('/unidadeCentral').then((resultUnid) => {
            if (response.data.data.length !== 0) {
              const defaultCentral = response.data.data.filter(
                (element: IDictionary) => element.CDFILIAL === actualCdCentral,
              );
              const cdfilcentpme = resultUnid.data.data.filter(
                (element: IDictionary) => element.CDAGREGADA === actualCdCentral,
              );
              if (defaultCentral.length !== 0) fieldCentral.setValue(defaultCentral[0]);
              else if (cdfilcentpme.length !== 0) fieldCentral.setValue(cdfilcentpme[0]);
              else fieldCentral.setValue(response.data.data[0]);
            } else msgParamCentral();
          });
        });
      } else {
        Http.post('/defaultCentral').then((response) => {
          if (response.data.dataset.central.length !== 0) {
            fieldCentral.datasource.get();
            fieldCentral.setValue(response.data.dataset.central[0]);
            store.commit('cdCentral', response.data.dataset.central[0].CDFILIAL);
          } else msgParamCentral();
        });
      }
    }
  }

  public setInfoCentral({ component }: IEventParam<Select>) {
    const cdCentral = component.value ? component.value.CDFILIAL : null;
    localStorage.setItem('CDFILIAL', cdCentral);
    store.commit('cdCentral', cdCentral);
    
    const header = Metadata.getInstance<IDictionary>('header');
    const filial = localStorage.CDFILIAL;
    const processDate = header.children.find((column: IDictionary) => column.name === 'processDate');
    
    if (!filial || filial === 'null') {
      processDate.text = '';
    } else {
      Http.post('/dataProcessamento', { filial }).then((response) => {
        const data = response.data.dataset.processDate[0];
        const format = (localStorage.getItem('LOGIN_LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY' : 'MM/DD/YYYY';
        processDate.text = moment(data, 'DD/MM/YYYY').format(format);
      });
    }
  }

  public getCdCentral() {
    return this.cdCentral;
  }

  public getNmCentral() {
    return this.nmCentral;
  }
  
  /* public async createModal() {
    this.confirmLogoutModal = await ModalService.createFromJsonPost('', { jsonPath: '/modals/confirmLogoutModal' });
  } */
  
  public setCentralMounted = () => {
    if (localStorage.getItem('LOGIN_TOKEN')) {
      const central = store.getters.cdCentral;
      const component = Metadata.getInstance<Select>('CDFILCENTPME');
      component.value = central;
      this.centralIsValid();
    }
  };
  
  public centralIsValid() {
    if (localStorage.getItem('LOGIN_TOKEN')) {
      const central = store.getters.cdCentral;
      if (central === null || central === 'null') {
        DialogService.hide();
        DialogService.show(
          {
            name: 'paramCentral',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::parametrizacaoCentral',
            buttons: [
              {
                name: 'dialogButton', label: 'pme::OK', component: 'ZdButton', events: { click: () => DialogService.hide() },
              },
            ],
          },
        );
      }
    }
  }
  
  public reloadDatasetCentral({ component }: IEventParam<Select>) {
    component.datasource.get();
  }

  get userName() {
    const username = localStorage.getItem('LOGIN_USER_NAME') || '';
    return this.getFormattedName(username);
  }

  get nameInitialLetter() {
    const name = this.userName;
    return name.slice(0, 1);
  }

  get logo() {
    return '/assets/logo.png';
  }

  public showMessageExpiredMenu() {
    TekLibAuth.showMessageExpiredMenu();
  }

  public closeMenu() {
    const menu = Metadata.getInstance<Menu>('main-menu');
    if (menu.opened && !menu.miniState) {
      menu.toggleMenu();
    }
  }

  public escAction() {
    AlertService.hide();
  }

  public async userInfoOnCreated() {
    const userInfoComponent = Metadata.getInstance<TekUserInfo>('userInfo');
    const currentModule = this.getCurrentModule();

    const { data } = await Http.get('/lib_generalInfo', { params: { currentModule } });
    userInfoComponent.versionInfo = data.versionInfo;

    if (data.userInfo.IMAGE) {
      this.hasImage = true;
      this.imagePath = data.userInfo.IMAGE;
    } else {
      this.hasImage = false;
    }
  }

  public userInfoOnLogoutClick() {
    TekLibAuth.logoutConfirm();
  }

  private getFormattedName(username: string): string {
    const usernameLowerCase = username.toLowerCase();
    const name = usernameLowerCase.replace(/(^\w{1})|(\s+\w{1})/g, (letter: string) => letter.toUpperCase());
    const splitName = name.split(' ');
    if (splitName.length > 1) {
      const firstName = splitName.shift();
      const lastName = splitName.pop();
      let middleName = '';

      splitName.forEach((item) => {
        middleName += ` ${item.slice(0, 1)}. `;
      });

      return `${firstName} ${middleName} ${lastName}`;
    }

    return name;
  }

  private getCurrentModule(): string {
    const { href } = window.location;
    let currentModule = 'product';
    if (href.includes('/modules/')) {
      let auxHref = href.split('modules/');
      auxHref = auxHref[1].split('/');
      [currentModule] = auxHref;
    }

    return currentModule;
  }
  
  public updateHeaderElements() {
    const isMobile = window.innerWidth < 780;
    this.headerElementsVisibility = !isMobile;
  }
}
