import {
  Loader, Metadata, IEventParam, IDictionary, Config, Datasource, Http,
} from '@zeedhi/core';
import {
  Select, GridEditable, Form, Button, LoadingService, DialogService,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme02400ClientesPorServicoController {
  public appController = Loader.getInstance('AppController');

  public crudController = Loader.getInstance('CrudController');

  public gridEditController = Loader.getInstance('GridEditableController');

  public utilities = Loader.getInstance('Utilities');
  
  private isFormatting: boolean = false;
  
  public clearFields() {
    const tpCardServ = Metadata.getInstance<Select>('TPCARDAPIOPORSERVICO');
    tpCardServ.setValue('');
  }
  
  public setTpCardServ({ component }: IEventParam<Select>) {
    const cdfilial = Metadata.getInstance<Select>('CDFILIAL');
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial.value ? cdfilial.value : null,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public reloadTelaClientesPorServico() {
    const formFiltro: Form = Metadata.getInstance<Form>('formFilter');
    const gridClientesPorServ = Metadata.getInstance<GridEditable>('gridClientes');
    
    if (formFiltro.validate() && !gridClientesPorServ.editing) {
      gridClientesPorServ.datasource.get();
    }
  }
  
  public setClientePorServico() {
    const addButton = Metadata.getInstance<Button>('addButton');
    const cdfilial = Metadata.getInstance<Select>('CDFILIAL');
    const tpCardServ = Metadata.getInstance<Select>('TPCARDAPIOPORSERVICO');
    const clientes = Metadata.getInstance<GridEditable>('gridClientes');
    
    if (!cdfilial.value || !tpCardServ.value) {
      clientes.datasource.data = [];
      addButton.disabled = true;
    } else {
      const valueTpCard: IDictionary = tpCardServ.value;
      const filter = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: cdfilial.value ? cdfilial.value : null,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: valueTpCard.CDTIPOCARD ? valueTpCard.CDTIPOCARD : '',
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: valueTpCard.CDSERVFILI ? valueTpCard.CDSERVFILI : '',
        },
      ];
      clientes.datasource.filter = filter;
      addButton.disabled = true;
      clientes.datasource.get().then(() => {
        addButton.disabled = false;
      });
    }
  }

  public saveCliServ() {
    const gridClientes: GridEditable = Metadata.getInstance('gridClientes');
    const rows: IDictionary[] = gridClientes.getEditedRows();

    if (rows.length === 0) {
      this.utilities.showAlert('alert', 'pme::nenhumaLinhaEditada', 'error');
      return;
    }

    const form = Metadata.getInstance<Form>('formFilter');
    const formClientes = Metadata.getInstance<Form>('formClientes');
    const unidade = form.value.CDFILIAL;
    const tpCardServ = form.value.TPCARDAPIOPORSERVICO;

    const savePromises: Promise<any>[] = [];

    if (gridClientes) gridClientes.datasource.loading = true;
    if (form.validate() && formClientes.validate()) {
      rows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDFILIAL = unidade;
        e.CDSERVFILI = tpCardServ.CDSERVFILI;
        e.CDTIPOCARD = tpCardServ.CDTIPOCARD;

        if (Array.isArray(e.NRINSJURCLIE)) {
          e.NRINSJURCLIE.forEach((item: any) => {
            const rowCopy = { ...e };
            if (item.CDCLIENTE) {
              rowCopy.CDCLIENTE = item.CDCLIENTE;
              const savePromise = this.saveRow(rowCopy);
              savePromises.push(savePromise.catch((error) => {
                const code = rowCopy.CDCLIENTE;
                return Promise.resolve(`CD: ${code} - ${error}`);
              }));
            }
          });
        }
      });

      Promise.all(savePromises)
        .then((results: any[]) => {
          const duplicateMessages: string[] = results.filter((result) => typeof result === 'string');
          if (duplicateMessages.length > 0) {
            this.utilities.showAlert('alert', 'pme::salvarItensDuplicados', 'success');
          } else {
            this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
          }
        })
        .catch((error: any) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        })
        .finally(() => {
          if (gridClientes) {
            this.gridEditController.defaultState(gridClientes);
            gridClientes.datasource.loading = false;
          }
        });
    }
  }

  // Função para salvar uma única linha
  private saveRow(row: IDictionary): Promise<void> {
    return this.crudController.saveRows([row], 'servcliepme', null);
  }

  public addRow({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formFilter');
    if (form.validate()) {
      this.gridEditController.addRow({ component });
    }
  }
  
  public getDynamicMask(value: string) {
    if (value.length <= 11) return '###.###.###-##';
    return '##.###.###/####-##';
  }
  
  public selecionaTipoDaMask(value: string) {
    const cpfMask: string = (Config.masks.cpfMask as PmeMask).pattern;
    const cnpjMask: string = (Config.masks.cnpjMask as PmeMask).pattern;
    const lengthCpf = 11;
    
    if (value.length <= lengthCpf) return cpfMask;
    return cnpjMask;
  }
  
  public changeCurrentCliServ({ component }: IEventParam<Datasource>) {
    if (!this.isFormatting) {
      this.isFormatting = true;
      setTimeout(() => {
        this.formatData({ component });
        this.isFormatting = false;
      }, 50);
    }
  }
  
  public async formatData({ component }: IEventParam<Datasource>) {
    if (!component.data || component.data.length === 0) return;
  
    const mappedData = component.data.map((row: IDictionary) => {
      const mask = this.selecionaTipoDaMask(row.NRINSJURCLIE);
      const NMRAZSOCCLIE = Array.isArray(row.NMRAZSOCCLIE) ? row.NMRAZSOCCLIE : [row.NMRAZSOCCLIE];
      const NRINSJURCLIE = Array.isArray(row.NRINSJURCLIE) ? row.NRINSJURCLIE : [this.applyMask(row.NRINSJURCLIE, mask)];
      return { ...row, NMRAZSOCCLIE, NRINSJURCLIE };
    });
    await component.updateData(mappedData);
  }

  private applyMask(value: string, mask: string): string {
    let index = 0;
    let maskedValue = '';
  
    Array.from(mask).forEach((char) => {
      if (char === 'N') {
        if (index < value.length) {
          maskedValue += value.charAt(index);
          index += 1;
        }
      } else {
        maskedValue += char;
      }
    });
  
    return maskedValue;
  }
  
  public confirmDeleteRazaoSociaç({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const rows = grid.selectedRows;
    if (rows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pme02400_excluiRazaoSoc',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteRowsRazaoSoc(rows, grid) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteRowsRazaoSoc(checkedRows: IDictionary, grid: GridEditable) {
    const buttonCancel = Metadata.getInstance<Button>('cancelButton');
    const buttonSave = Metadata.getInstance<Button>('saveButton');
    LoadingService.show(' ');
    Http.post('/deleteRazaoSocial', { checkedRows }).then(() => {
      grid.cancelEditedRows();
      grid.datasource.get();
      grid.selectedRows = [];
      buttonCancel.disabled = true;
      buttonSave.disabled = true;
      this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'green');
      DialogService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      DialogService.hide();
    }).finally(() => {
      LoadingService.hide();
    });
  }
}
